//======================================================= file = common.h =====
//=  Include file for Khotimsky PPS (with bufferless demultiplexors)          =
//=============================================================================
//=  Notes:  1) This file contains                                            =
//=               - Constants                                                 =
//=               - Common (external) variables                               =
//=               - Common (external) CSIM types                              =
//=               - Function prototypes                                       =
//=          2)  Basic time unit is the cell transmission time                =
//=---------------------------------------------------------------------------=
//=  History: AAS (04/01/03) - Genesis                                        =
//=============================================================================
#pragma warn -pro                     // Disable function prototype warning
#pragma warn -stu                     // Disable undefined structure warning
#pragma warn -rch                     // Disable unreachable code warning
#pragma warn -ccc                     // Disable condition always true warning

//----- Conditional definitions -----------------------------------------------
#if defined(MAIN)                     // Determine if main or external
  #define EXTERN
#endif
#if defined(EXTERNAL)
  #define EXTERN extern
#endif

//----- Constants -------------------------------------------------------------
#define VERSION  "1.00 <04/01/03>"    // Version number
#define OUT_FILE      "result.txt"    // Name of output file

#define FALSE             0    // FALSE = 0
#define TRUE              1    // TRUE = 1
#define GEN_FLAG          1    // 1 for Bernoulli, 2 for IBP
#define N                16    // Number of input ports in the switch
#define K                19    // Number of switching planes in PPS
#define r                10    // Time to transmit a cell on internal links
#define S               1.9    // Internal speed-up
#define CELL_TIME       1.0    // Time to transmit a cell
#define FLOW_CELLS  1000000    // Maximum # of cells of an inport-outport flow

//----- External variables for generate1() ------------------------------------
EXTERN double    Lambda;       // Probability of a cell in a slot

//----- External variables for generate2() ------------------------------------
EXTERN double    P;            // Pr[on at t+1 | on at t]
EXTERN double    Q;            // Pr[off at t+1 | off at t]
EXTERN double    Alpha;        // Pr[arrival at t | on]

//----- Overall external variables --------------------------------------------
EXTERN int       Outport_count[N]; // Counter semaphores for outport
EXTERN int       Next_plane[N][N]; // Switch plane number
EXTERN double    Sim_time;         // Simulation time
EXTERN int       Demux_loss[N];    // Cell loss at demultiplexor
EXTERN int       Mux_loss[N];      // Cell loss at multiplexor 
EXTERN int       Ail[K][N];        // Available input link set
EXTERN double    Ail_freetime[K][N];  // Time,when internal input link be free
EXTERN int       Tag[N][N];        // Sequence # attached to a cell
EXTERN int       Count;            // # of served cells of a flow
EXTERN int       Reorder;          // # of re-order cells of a flow
EXTERN int       Index;            // Index of next in-order cell of a flow
EXTERN short     Cell_served[FLOW_CELLS]; // Status of cells of a flow

//----- CSIM Types ------------------------------------------------------------
EXTERN FACILITY  Outport_internal[K][N]; // Facility for internal outports
EXTERN FACILITY  Channel_server[K][N]; // Facility for internal links at demux
EXTERN FACILITY  Outport_external[N];  // Facility for external outports
EXTERN EVENT     Mux_Q_event[K][N];    // Queuing Event at multiplexor buffers
EXTERN TABLE     Delay_table[N];       // Delay table for cell delay
EXTERN MBOX      Seq_number[K][N];   // Mailbox to send timestamp to mux_sel()

//----- Function prototypes ---------------------------------------------------
// Traffic generators
EXTERN void generate1(int in_id);
EXTERN void generate2(int in_id);
EXTERN void (* gen[N])(int in_id);

// Internal channel Prototypes
EXTERN void channel(int in_id,int out_id,
int channel_num,int timestamp,int tag,double org_time);
EXTERN void (* chan[K][N])(int in_id,int out_id,
int channel_num,int timestamp,int tag,double org_time);

// Input Demultiplexors
EXTERN void demux(int in_id,int out_id,double org_time);
EXTERN void (* indemux[N])(int in_id,int out_id,double org_time);

// Output Multiplexors
EXTERN void mux(int from_id,int out_id,
int channel_num,int timestamp,int tag,double org_time);
EXTERN void (* outmux[N])(int from_id,int out_id,
int channel_num,int timestamp,int tag,double org_time);

// Multiplexors Selector
EXTERN void mux_sel(int out_id);
EXTERN void (* sel[N])(int out_id);

// Input ports
EXTERN void inport(int in_id,int out_id,
int channel_num,int timestamp,int tag,double org_time);
EXTERN void (* in[K][N])(int in_id,int out_id,
int channel_num,int timestamp,int tag,double org_time);

// Output ports
EXTERN void outport(int from_id,int out_id,
int channel_num,int timestamp,int tag,double org_time);
EXTERN void (* out[K][N])(int from_id,int out_id,
int channel_num,int timestamp,int tag,double org_time);

// Run-time simulation status reporter
EXTERN void run_sim(double run_time, int run_num);
