//================================================ file = demux.c =============
//=  Demultiplexor for Khotimsky PPS switch simulation model                  =
//=---------------------------------------------------------------------------=
//=  Contains: demux()
//=============================================================================
//=  Notes:  1) Traffic dispatch algorithm is least recently used             =
//=---------------------------------------------------------------------------=
//=  History: AAS (04/01/03) - Genesis                                        =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include <assert.h>     // Needed for assert() macro
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff
//=============================================================================
void demux(int in_id,int out_id,double org_time)
{

  int  i,j;         // Loop Counters 
  int  channel_num; // Internal switch plane #, to dispatch a cell 
  int  temp_flag;   // Boolean flag
  int  timestamp;   // Cell arrival timestamp to switch

  create("demux");

  // Drop the extra cells of an inport-outport flow
  if(Tag[in_id][out_id]>=FLOW_CELLS)
  {
    return;
  }

  // Checking for the free internal inport links
  for(i=0;i<K;i++)
  {
    if(Ail[i][in_id] == FALSE)
    {
      if(clock >= Ail_freetime[i][in_id])
      {
        Ail[i][in_id] = TRUE;
      }
    }
  }

  // Finding the least recently used internal input link (by a flow)
  temp_flag = FALSE;
  j  = Next_plane[in_id][out_id];
  for(i=0;i<K;i++)
  {
    if(Ail[(j+i)%K][in_id] == TRUE)
    {
      channel_num = (j+i)%K;
      Ail[channel_num][in_id] = FALSE;
      Ail_freetime[channel_num][in_id] = clock +(r*CELL_TIME);
      Next_plane[in_id][out_id] = (channel_num+1)%K;
      assert(channel_num<K);
 
      // Send cell to an internal channel queue at demux
      timestamp = (int)clock;
      chan[channel_num][in_id](in_id,out_id,channel_num,
                               timestamp,Tag[in_id][out_id],org_time);
      Tag[in_id][out_id]++;
      temp_flag  = TRUE;
      break;
    } 
  }
  // Assert, if all internal inport links are busy 
  assert(temp_flag);
}

//=============================================================================
//==  Function for an internal input channel at demux                        ==
//=============================================================================
void channel(int in_id,int out_id,
int channel_num,int timestamp,int tag,double org_time)
{
  create("channel");

  // Drop the cell, if Queue is full
  if(qlength(Channel_server[channel_num][in_id])>0)
  {
    Demux_loss[in_id]++;
    return;
  }

  // Queue cell on the channel queue
  reserve(Channel_server[channel_num][in_id]);
  hold(r * CELL_TIME);
  release(Channel_server[channel_num][in_id]);

  // Forward cell to inport of the selected switch plane
  in[channel_num][in_id](in_id,out_id,channel_num,timestamp,tag,org_time);
  return;
}
