//===================================================== file = generate.c =====
//=  Traffic generator for VOQ switch simulation model                        =
//=============================================================================
//=  Notes:  1)                                                               =
//=---------------------------------------------------------------------------=
//=  Contains: generate1() - Bernoulli arrivals                               =
//=            generate2() - Interrupted Bernoulli arrivals                   =
//=---------------------------------------------------------------------------=
//=  History:  KJC (11/15/00) - Genesis                                       =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

//=============================================================================
//=  Traffic generator for Bernoulli arrivals                                 =
//=============================================================================
void generate1(int in_id)
{
  double z;             // Uniform RV from 0.0 to 1.0
  double org_time;      // Origination time of a cell
  int    out_id;        // Destination outport number

  create("generate1");

  // Do forever
  while(TRUE)
  {
    // Hold for one cell time
    hold(CELL_TIME);

    // Determine if there is to be a cell in this slot
    z = uniform(0.0, 1.0);
    if (z <= Lambda)
    {
      // Generate output port id and origination time
      out_id = random_int(0, N - 1);
      org_time = clock;
      indemux[in_id](in_id, out_id, org_time);
    }
  }
}

//=============================================================================
//=  Traffic generator for Interrupted Bernoulli (IBP) arrivals               =
//=============================================================================
void generate2(int in_id)
{
  double z;             // Uniform RV from 0.0 to 1.0
  double org_time;      // Origination time of a cell
  int    out_id;        // Destination outport number
  int    on_flag;       // Flag for on or off

  create("generate2");

  // Do forever (starting in an "on" period)
  on_flag = TRUE;
  while(TRUE)
  {
    // Loop for "on" case (all cells in a burst go to one output)
    out_id = random(0, (N - 1));
    while (on_flag == TRUE)
    {
      z = uniform(0.0, 1.0);
      if (z <= Alpha)
      {
        org_time = clock;
        indemux[in_id](in_id, out_id, org_time);
      }
      hold(CELL_TIME);

      z = uniform(0.0, 1.0);
      if (z > P) on_flag = FALSE;
    }

    // Loop for "off" case
    while (on_flag == FALSE)
    {
      hold(CELL_TIME);
      z = uniform(0.0, 1.0);
      if (z > Q) on_flag = TRUE;
    }
  }
}

